#include<stdio.h> //For Standard I/O
#include<stdlib.h> //For Functions like ATOI
#include<ctype.h> //For Type check
#include<string.h> //For String manipulation
#include<math.h> //For Math functions
#include<fitsio.h>

#include "iolib.h"

#define MAX_SIZE 8192

int readimage(char *filename, long naxes[2], float *data) {
    fitsfile *fptr;       /* pointer to the FITS file, defined in fitsio.h */
    int status,  nfound, anynull;
    long nbuffer, npixels, ii;
    float nullval;
    float *buffer;
    long fpixel[2],lpixel[2],initialpixel;
    int j;  
 
    status = 0;

    if ( fits_open_file(&fptr, filename, READONLY, &status) )
	printerror(status);
    if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status) )
	printerror(status);
    fpixel[0]=1;
    fpixel[1]=0;
    if (naxes[0]>MAX_SIZE) {
      fpixel[0]=(int)(naxes[0]-MAX_SIZE)/2;
    }
    if (naxes[1]>MAX_SIZE) {
      fpixel[1]=(int)(naxes[1]-MAX_SIZE)/2;
    }
    lpixel[0]=fpixel[0]+naxes[0];
    lpixel[1]=fpixel[1]+naxes[1];
    npixels  = naxes[0]*naxes[1]; /* number of pixels in the image */
    nullval  = 0;                /* don't check for null values in the image */
    initialpixel=fpixel[0]+fpixel[1]*naxes[0];
     ii=0;


    buffer = (float *)malloc(naxes[0]*sizeof(float));
    data = (float *)malloc(npixels*sizeof(float));
    while(npixels>0)  {
	nbuffer=npixels;
	if (npixels>naxes[0])
	    nbuffer=naxes[0];
	
	if(fits_read_img(fptr,TFLOAT,initialpixel,nbuffer,&nullval,buffer,&anynull,&status))
	printerror(status);
	
	for (j=0;j<nbuffer;j++){
	    if ((buffer[j])!=(1.0*buffer[j])) {
		data[ii+j]=0.0;
	    } else {
		data[ii+j]=buffer[j];
	    }
	}
	ii+=nbuffer;
	npixels-=nbuffer;
	initialpixel += naxes[0];
    }
    
    
   if ( fits_close_file(fptr, &status) )
       printerror(status);
    

    
    return(status);
}

int writeimage(float* data, char *filename, long naxes[2]) {
    fitsfile *fptr;
    int status, ii, jj;
    long fpixel,nelements;
    int bitpix=FLOAT_IMG;
    long naxis=2;
    float *local_data[naxes[1]];
    local_data[0] = (float*) malloc(naxes[0]*naxes[1] * sizeof (float));
    for (jj=1;jj<naxes[1];jj++) {
	local_data[jj]=local_data[jj-1]+naxes[0];
    }
    for (jj=0;jj<naxes[1];jj++) {
	for (ii=0;ii<naxes[0];ii++) {
	    local_data[jj][ii]=data[ii+jj*naxes[0]];
	}
    }
    remove(filename);
    if(fits_create_file(&fptr,filename,&status))
	printerror(status);
    if (fits_create_img(fptr,bitpix,naxis,naxes,&status))
	printerror(status);
    
    fpixel=1;
    nelements=naxes[0]*naxes[1];  
    if (fits_write_img(fptr,TFLOAT,fpixel,nelements,local_data[0],&status))
	printerror(status);
    
    if (fits_close_file(fptr,&status))
	printerror(status);
    
    free(local_data[0]);
    return(status);
}


void printerror(int status) {
  if (status) {
    fits_report_error(stderr,status);
    exit(status);
  }
  return;
}


int readimage(char *filename, long naxes[2], float *data) {
    fitsfile *fptr;       /* pointer to the FITS file, defined in fitsio.h */
    int status,  nfound, anynull;
    long nbuffer, npixels, ii;
    float nullval;
    float *buffer;
    long fpixel[2],lpixel[2],initialpixel;
    int j;  
 
    status = 0;

    if ( fits_open_file(&fptr, filename, READONLY, &status) )
	printerror(status);
    if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status) )
	printerror(status);
    fpixel[0]=1;
    fpixel[1]=0;
    if (naxes[0]>MAX_SIZE) {
      fpixel[0]=(int)(naxes[0]-MAX_SIZE)/2;
    }
    if (naxes[1]>MAX_SIZE) {
      fpixel[1]=(int)(naxes[1]-MAX_SIZE)/2;
    }
    lpixel[0]=fpixel[0]+naxes[0];
    lpixel[1]=fpixel[1]+naxes[1];
    npixels  = naxes[0]*naxes[1]; /* number of pixels in the image */
    nullval  = 0;                /* don't check for null values in the image */
    initialpixel=fpixel[0]+fpixel[1]*naxes[0];
     ii=0;


    buffer = (float *)malloc(naxes[0]*sizeof(float));
    data = (float *)malloc(npixels*sizeof(float));


    while(npixels>0)  {
	nbuffer=npixels;
	if (npixels>naxes[0])
	    nbuffer=naxes[0];
	
	if(fits_read_img(fptr,TFLOAT,initialpixel,nbuffer,&nullval,buffer,&anynull,&status))
	printerror(status);
	
	for (j=0;j<nbuffer;j++){
	    if ((buffer[j])!=(1.0*buffer[j])) {
		data[ii+j]=0.0;
	    } else {
		data[ii+j]=buffer[j];
	    }
	}
	ii+=nbuffer;
	npixels-=nbuffer;
	initialpixel += naxes[0];
    }
    
    
   if ( fits_close_file(fptr, &status) )
       printerror(status);
    

    
    return(status);
}
